@@ -25,8 +25,8 @@ def SchedulerMethod(*methodArgs, requestClass=None, **methodKwargs) :
         methodClassName = ReflectionHelper.getMethodClassName(resourceInstanceMethod)
         methodName = ReflectionHelper.getName(resourceInstanceMethod)
         shedulerId = methodKwargs.get('id', f'{methodClassName}{c.DOT}{methodName}')
-        instancesUpTo =  methodKwargs.get('instancesUpTo', 1)
-        day_of_week = methodKwargs.get('daysOfWeek', None)
+        instancesUpTo =  methodKwargs.pop('instancesUpTo', 1)
+        day_of_week = methodKwargs.pop('daysOfWeek', None)
         methodKwargs['id'] = shedulerId
         methodKwargs['max_instances'] = instancesUpTo
         methodKwargs['day_of_week'] = day_of_week
