@@ -67,6 +67,8 @@ def getControllerList(resourceName, resourceModuleName) :
         resource = globals.importResource(controllerName, resourceModuleName=resourceModuleName)
         if resource :
             importedControllerList.append(resource)
+    if 0 == len(importedControllerList) :
+        raise Exception(f'Not possible to import {resourceName} controller')
     return importedControllerList
 
 @Function
@@ -150,6 +152,11 @@ def addServiceListTo(apiInstance,serviceList) :
     for service in serviceList :
         apiInstance.bindResource(apiInstance,service())
 
+@Function
+def addSchedulerListTo(apiInstance,schedulerList) :
+    for scheduler in schedulerList :
+        apiInstance.bindResource(apiInstance,scheduler())
+
 @Function
 def addClientListTo(apiInstance,clientList) :
     for client in clientList :
@@ -199,6 +206,7 @@ def addFlaskApiResources(
         appInstance,
         jwtInstance,
         controllerList,
+        schedulerList,
         serviceList,
         clientList,
         repositoryList,
@@ -217,6 +225,7 @@ def addFlaskApiResources(
     addRepositoryTo(apiInstance, repositoryList, model)
     addServiceListTo(apiInstance, serviceList)
     addClientListTo(apiInstance, clientList)
+    addSchedulerListTo(apiInstance, schedulerList)
     addControllerListTo(apiInstance, controllerList)
     addValidatorListTo(apiInstance, validatorList)
     addMapperListTo(apiInstance, mapperList)
